/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#include <initng.h>


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <initng_handler.h>
#include <initng_global.h>
#include <initng_plugin_hook.h>
#include <initng_common.h>
#include <initng_toolbox.h>
#include <initng_static_data_id.h>
#include <initng_static_states.h>

s_entry FILES_TO_EXIST = { "require_file", STRINGS, NULL,
    "Check so that this files exits before launching."
};
s_entry FILES_TO_EXIST_AFTER = { "files_to_exist_after", STRINGS, NULL,
    "Make sure this files exits after service is launched, or mark failed."
};


/*
 * Do this check before START_DEP_MET can be set.
 */
static int check_files_to_exist(active_db_h * service)
{
    const char *file = NULL;
    struct stat file_stat;

    while ((file =
            initng_active_db_get_next_string(&FILES_TO_EXIST, service, file)))
    {
        D_("Service %s need file %s to exist\n", service->name, file);
        if (stat(file, &file_stat) != 0)
        {
            D_("File %s needed by %s doesn't exist.\n", file, service->name);
            /* set the alarm, make sure initng will search files, in one second */
            initng_global_set_sleep(1);

            /* don't change status of service to START_DEP_MET */
            return (FALSE);
        }
    }
    return (TRUE);
}

/*
 * Do this test before status RUNNING can be set.
 */
static int check_files_to_exist_after(active_db_h * service)
{
    const char *file = NULL;
    struct stat file_stat;

    while ((file =
            initng_active_db_get_next_string(&FILES_TO_EXIST_AFTER, service,
                                             file)))
    {
        D_("Service %s need file %s before it can be set to RUNNING\n",
           service->name, file);
        if (stat(file, &file_stat) != 0)
        {
            D_("File %s needed by %s doesn't exist.\n", file, service->name);
            /* set the alarm, make sure initng will search files, in one second */
            initng_global_set_sleep(1);

            /* don't change status of service to START_READY */
            return (FALSE);
        }
    }
    return (TRUE);
}

int module_init(int api_version)
{
    S_;
    if (api_version != API_VERSION)
    {
        F_("This module is compiled for api_version %i version and initng is compiled on %i version, won't load this module!\n", INITNG_VERSION, api_version);
        return (FALSE);
    }

    initng_service_data_types_add(&FILES_TO_EXIST);
    initng_service_data_types_add(&FILES_TO_EXIST_AFTER);
    initng_plugin_hook_add(&g.START_DEP_MET, 55, &check_files_to_exist);
    initng_plugin_hook_add(&g.UP_MET, 55, &check_files_to_exist_after);
    return (TRUE);
}

void module_unload(void)
{
    S_;
    initng_service_data_types_del(&FILES_TO_EXIST);
    initng_service_data_types_del(&FILES_TO_EXIST_AFTER);
    initng_plugin_hook_del(&g.START_DEP_MET, &check_files_to_exist);
    initng_plugin_hook_del(&g.UP_MET, &check_files_to_exist_after);
}
