
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _GNU_SOURCE
#include <fcntl.h>
#include <sys/file.h>
#include <sys/types.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>
#include <getopt.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <assert.h>
#include <errno.h>

/* #include "ngc.h" */
#include <initng.h>
#include <initng_global.h>
#include <initng_control_command.h>
#include <initng_active_state.h>
#include <initng_toolbox.h>
#include <initng-paths.h>

#include "libngcclient.h"

#include "ansi_colors.h"

int debug = FALSE;

int header_printed = FALSE;
int quiet = FALSE;

#define print_out(...); { if(quiet==FALSE) printf(__VA_ARGS__ ); }

#define C_ERROR "\n  "C_FG_RED" [ERROR] -->> "C_OFF

#define TMP_QUIET if(quiet==FALSE) quiet=3;
#define TMP_UNQUIET if(quiet==3) quiet=FALSE;

static int send_and_handle(const char c, const char * l, const char * opt)
{
    char *string = NULL;
    printf("send_and_handle(%c, %s, %s);\n", c, l, opt);

    string=ngc_send_and_get_string(c, l, opt, FALSE);
    printf("\n\n%s\n", string);
    free(string);
    return(TRUE);
}

/* THIS IS MAIN */
int main(int argc, char *argv[])
{
    int i;
    int cc = 1;
    char *Argv=NULL;

    assert(argv[0]);

    /*
     * Only on first input from initng, we will print a
     * initng header with version info, after then 
     * header_printed is true, and probits this.
     */
    header_printed = FALSE;

    /* Look if --quite is along options */
    for(i=0;i<argc;i++)
    {
	if (argv[i] && strcmp(argv[i], "--quiet") == 0)
	    quiet = TRUE;
    }
    
    /* print banner */
    print_out(C_FG_LIGHT_BLUE
              "\n Next Generation init Control. version ( %s )" C_OFF "\n",
              VERSION);
    print_out(C_FG_NEON_GREEN " http://initng.thinktux.net" C_OFF "\n");
    print_out(" Author: Jimmy Wennlund <jimmy.wennlund@gmail.com>\n\n");


    /*
     * Skip path in Argv.
     * example argv[0] == "/sbin/ngc" then Argv == "ngc"
     * example argv[0] == "./ngstart" then Argv == "ngstart"
     */
    {
        i = 0;

	/* skip to last char of "/sbin/ngc" */
        while (argv[0][i])
            i++;

	/* then walk backwards, to the first '/' found */
        while (i > 0 && argv[0][i] != '/')
        {
            i--;
        }

	/* Here we have the program name */
        Argv = &argv[0][i];

	/* Make sure there is no stale '/' char in beginning */
        if (Argv[0] == '/')
            Argv++;

	/*
	 * Special case, libtool test adds lt-ngc to path
	 * when running in make dir.
	 * So translate "lt-ngc" to "ngc"
	 */
	if (Argv[0] == 'l' && Argv[1] == 't' && Argv[2] == '-')
	    Argv+=3;
	    
        /* Only for testing.
	 * printf("Argv: %s argv[0]: %s\n", Argv, argv[0]);
	 */
    }

    /* check if cmd-line contains "ngdc", if set debug */
    if (strncasecmp(Argv, "ngd", 3) == 0)
    {
        print_out(C_FG_YELLOW "Warning. This is ngdc!" C_OFF "\n");
        debug = TRUE;
    }


    if (debug == FALSE && getuid() != 0)
    {
        printf(C_ERROR "You need root access to communicate with initng."
               C_OFF "\n");
        exit(2);
    }

    /* check that Argc string is not ngc, NGC, ngdc, NGDC */
    if (strcasecmp(Argv, "ngc") != 0 && strcasecmp(Argv, "ngdc") != 0 &&
	strcasecmp(Argv, "ngc4") != 0 && strcasecmp(Argv, "ngdc4") != 0)
    {
        /*printf("Not standard ngc or ngdc command.\n"); */
        /* check if argv0 starts with "ng..." */
        if (Argv[0] == 'n' && Argv[1] == 'g' && Argv[2])
        {
            /* if its ngstart, call send_and_handle("start"); */
            if (Argv[2] == 'd' && Argv[3])
            {
                /* check if is a string or char */
                if (Argv[4])
                    return (send_and_handle('\0', &Argv[3], argv[1]));
                else
                    return (send_and_handle(Argv[3], NULL, argv[1]));
            }
            else
            {
                /* check if its a string or char */
                if (Argv[3])
                    return (send_and_handle('\0', &Argv[2], argv[1]));
                else
                    return (send_and_handle(Argv[2], NULL, argv[1]));
            }
        }

        /* else call standard */
        return (send_and_handle('\0', Argv, argv[1]));
    }

    /* make sure there are any arguments at all */
    if (argc <= 1)
    {
        send_and_handle('h', NULL, NULL);
        exit(0);
    }

    /* walk thru all arguments */
    while (argv[cc])
    {
        char *opt = NULL;

        /* every fresh start needs a '-' char */
        if (argv[cc][0] != '-')
        {
    	    send_and_handle('h', NULL, NULL);
            exit(1);
        }

        /* check that there is a char after the '-' */
        if (!argv[cc][1])
        {
    	    send_and_handle('h', NULL, NULL);
            exit(1);
        }

        /* if next option contains data, but not starting with an '-', its considered an option */
        if (argv[cc + 1] && argv[cc + 1][0] != '-')
            opt = argv[cc + 1];


        /* if it is an --option */
        if (argv[cc][1] == '-')
        {
            if (!send_and_handle('\0', &argv[cc][2], opt))
                exit(1);
        }
        else
        {
            if (!send_and_handle(argv[cc][1], NULL, opt))
                exit(1);
        }

        cc++;
        if (opt)
            cc++;


    }
    printf("\n\n");
    exit(0);
}
