
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _GNU_SOURCE
#include <fcntl.h>
#include <sys/file.h>
#include <sys/types.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>
#include <getopt.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <assert.h>
#include <errno.h>
#include <poll.h>

#include <initng.h>
#include <initng-paths.h>

#include "libngeclient.h"
int main(int argc, char *argv[]);

static void connected(nge_event *e, int pver, char *initng_version)
{
    fprintf(stdout,"Connected to initng %s\n", initng_version);
}

static void dissconnected(nge_event *e)
{
    fprintf(stdout,"Dissonnected from initng.\n");
}

static void service_change(nge_event * e, char * service, e_is is, char * state)
{
    switch(is)
    {
	case IS_UP:
	case IS_DOWN:
	case IS_FAILED:
	case IS_STARTING:
	case IS_STOPPING:
	case IS_WAITING:
	default:
	    fprintf(stdout,"Service \"%s\" have state \"%s\" (%i)\n", service, state, is);
	    break;
    }
}

static void ping(nge_event *e)
{
    fprintf(stdout,"Got an ping from initng.");
}

static void service_output(nge_event *e, char * service, char * process, char * output)
{
    fprintf(stdout,"Service \"%s\" process \"%s\" outputed:\n%s\n", service, process, output);
}

static void err_msg(nge_event *e, e_mt mt, char * file, char * func, int line, char * message)
{
    fprintf(stdout,"Message mt: %i, file: %s, func: %s, line %i.\n%s\n",
	mt, file, func, line, message);
}

static void sys_state(nge_event * e, h_sys_state state)
{
    switch(state)
    {
    	case STATE_STARTING:
    	case STATE_UP:
    	case STATE_STOPPING:
    	case STATE_ASE: /* All Services Exited */
    	case STATE_SERVICES_LOADED:  /* This should only be changed by sys_state_services_loaded, that resets sysstate to last service afterwards. */
    	case STATE_EXIT:
    	case STATE_RESTART:
    	case STATE_SULOGIN:
    	case STATE_HALT:
    	case STATE_REBOOT:
    	case STATE_EXECVE:
	default:
	    fprintf(stdout,"Initng got a new system state no: %i\n", state);
	    break;
    }
}

/* THIS IS MAIN */
int main(int argc, char *argv[])
{
    nge_event e;
    
    /* Always null this, to reset it */
    memset(&e, 0, sizeof(e));
    
    /* set the hooks */
    e.connect = &connected;
    e.dissconnect = &dissconnected;
    e.service_state_change = &service_change;
    e.system_state_change = &sys_state;
    e.initial_service_state = &service_change;
    e.initial_system_state = &sys_state;
    e.service_output = &service_output;
    e.err_msg = &err_msg;
    e.ping = &ping;
    
    /* open correct socket */
    if(strcmp(argv[0], "ngde")==0)
	ngeclient_open_socket(&e, NGE_TEST);
    else
	ngeclient_open_socket(&e, NGE_REAL);

    /* if open_socket fails, ngeclient_error is set */
    if(ngeclient_error)
    {
	fprintf(stderr, "%s\n", ngeclient_error);
	exit(1);
    }
    
    /* this is the MAIN LOOP for ngeclient */
    ngeclient_exec(&e);

    /* check for failures */
    if(ngeclient_error)
    {
	fprintf(stderr, "%s\n", ngeclient_error);
    }
    
    /* clean up */
    ngeclient_close_socket(&e);
    exit(0);
}
