
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#ifndef SERVICE_DB_H
#define SERVICE_DB_H

#include "initng_list.h"
#include "initng_service_types.h"
#include "initng_struct_data.h"

typedef struct s_service_cache_h service_cache_h;
struct s_service_cache_h
{
    char *name;
    stype_h *type;

    /* list container, to store data */
    s_data data;

    /* father/relative data */
    char *father_name;
    service_cache_h *father;

    /* add a list entry */
    struct list_head list;
};

/* service handling functions */
int initng_service_cache_add(service_cache_h * s);
service_cache_h *initng_service_cache_copy(char *name, service_cache_h * s);
service_cache_h *initng_service_cache_new(const char *name, stype_h * type);

/* Searching */
service_cache_h *initng_service_cache_find_by_exact_name(const char *service);
service_cache_h *initng_service_cache_find_by_name(const char *service);
service_cache_h *initng_service_cache_find_in_name(const char *service);

int initng_service_cache_find_father(service_cache_h * service);

/* freeing */
int initng_service_cache_free(service_cache_h * to_free);
void initng_service_cache_free_all(void);

/* update */
void initng_service_cache_change(service_cache_h * from,
                                 service_cache_h * to);

/* standard wrappers using service */
#define initng_service_cache_get_next(t,s,l) d_get_next(t, &(s)->data.list, l)

/* statics */
#define initng_service_cache_is(t,s) initng_service_cache_is_var(t,NULL,s)
#define initng_service_cache_get_string(t,s) initng_service_cache_get_string_var(t, NULL, s)
#define initng_service_cache_get_int(t,s) initng_service_cache_get_int_var(t, NULL, s)
#define initng_service_cache_get_next_string(t,s,l) initng_service_cache_get_next_string_var(t,NULL,s,l)

#define initng_service_cache_set_string(type, from_service, string)  d_set_string(type, &from_service->data.list, string)
#define inting_service_cache_set_another_string(type, from_service, string) d_set_another_string(type, &from_service->data.list, string)
#define initng_service_cache_set_int(type, from_service, value) d_set_int(type, &from_service->data.list, value)
#define initng_service_cache_set(type, from_service) d_set(type, &from_service->data.list)
#define initng_service_cache_remove(type, from_service) d_remove(type, &from_service->data.list)

/* variable */
int initng_service_cache_is_var(s_entry * type, const char *vn,
                                service_cache_h * s);
const char *initng_service_cache_get_string_var(s_entry * type,
                                                const char *vn,
                                                service_cache_h * s);
int initng_service_cache_get_int_var(s_entry * type, const char *vn,
                                     service_cache_h * s);
const char *initng_service_cache_get_next_string_var(s_entry * type,
                                                     const char *vn,
                                                     service_cache_h * s,
                                                     const char *last);

#define initng_service_cache_set_string_var(type, vn, from_service, string)  d_set_string_var(type, vn, &from_service->data.list, string)
#define initng_service_cache_set_another_string_var(type, vn, from_service, string) d_set_another_string_var(type, vn, &from_service->data.list, string)
#define initng_service_cache_set_int_var(type, vn, from_service, value) d_set_int_var(type, vn, &from_service->data.list, value)
#define initng_service_cache_set_var(type, vn, from_service) d_set_var(type, vn, &from_service->data.list)
#define initng_service_cache_remove_var(type, vn, from_service) d_remove_var(type, vn, &from_service->data.list)


/* global */
#define initng_service_cache_remove_all(from_service) d_remove_all(&from_service->data.list)
#define initng_service_cache_count_type(type, from_service) d_count_type(type, &from_service->data.list)

/* service_list walker */
#define while_service_cache(current) list_for_each_entry_prev(current, &g.service_cache.list, list)
#define while_service_cache_safe(current, safe) list_for_each_entry_prev_safe(current, safe, &g.service_cache.list, list)

#endif
